/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.2 Baustellensimulation
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.simbaustelle.simbaustelle;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.simbaustelle.verwaltung.BaustellenDatenSchnittstelle;
import de.bsvrz.ibv.simbaustelle.verwaltung.BaustellenSchnittstelle;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnitt;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnittAllgemein;

/**
 * Virtuelles Baustellenobjekt zum Test des Algorithmus der Stausimulation.<br>
 * Instanzen dieser Klasse repräsentieren eine Standard-Testbaustelle mit zwei
 * Fahrstreifen, die mit {@link TestBaustellenDaten} versorgt werden. Lediglich
 * die Anfangs- und Endzeit der Baustelle können angepasst werden.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class TestBaustelle implements BaustellenSchnittstelle {

	/**
	 * die daten, die die Baustelle beschreiben.
	 */
	private final TestBaustellenDaten daten = new TestBaustellenDaten();

	/**
	 * Für die Testausführug wird immer eine Baustelle mit 2 Fahrstreifen
	 * angenommen.
	 *
	 * @see de.bsvrz.ibv.simbaustelle.verwaltung.BaustellenSchnittstelle#anzahlFahrStreifen(long)
	 */
	@Override
	public final int anzahlFahrStreifen(final long position) {
		return 2;
	}

	/**
	 * Die Funktion liefert hier einen vordefinierten Datensatz mit den
	 * konfigurierenden Daten der Baustelle.
	 *
	 * @see de.bsvrz.ibv.simbaustelle.verwaltung.BaustellenSchnittstelle#getDaten()
	 */
	@Override
	public final BaustellenDatenSchnittstelle getDaten() {
		return daten;
	}

	/**
	 * liefert die Endzeit der Baustelle.
	 *
	 * @return den Zeitpunkt
	 */
	public long getEndZeit() {
		long result = Long.MAX_VALUE;
		result = daten.getEnde();

		return result;
	}

	/**
	 * Der vor einer Baustelle liegende Messquerschnitt wird lediglich zur
	 * Bestimmung des zufließenden Verkahrs benötigt. Da dieser ebenfalls durch
	 * eine Testimplementierung der entsprechenden Schnittstelle ermittelt wird,
	 * wird ein Messquerschnitt an dieser Stelle nicht benötigt und die Funktion
	 * liefert immer den Wert <b><i>null</i></b>.
	 *
	 * @see de.bsvrz.ibv.simbaustelle.verwaltung.BaustellenSchnittstelle#getMessquerschnittDavor()
	 */
	@Override
	public final MessQuerschnitt getMessquerschnittDavor() {
		return null;
	}

	/**
	 * Es wird immer der Name "Testbaustelle" geliefert.
	 */
	@Override
	public final String getName() {
		return "Testbaustelle";
	}

	@Override
	public SystemObject getObjekt() {
		// kein Objekt, da nur eine interne Berechnung verwendet wird
		return null;
	}

	/**
	 * liefert die Restkapazität der Baustelle zum angegebenen Zeitpunkt.
	 *
	 * @param zs
	 *            der Zeitpunkt
	 * @return die Restkapazität
	 */
	public long getRestKapazitaet(final long zs) {
		long result = Long.MAX_VALUE;
		result = daten.getRestKapazitaet(zs);

		return result;
	}

	/**
	 * liefert die Startzeit der Baustelle.
	 *
	 * @return die Zeit
	 */
	public long getStartZeit() {
		long result = Long.MAX_VALUE;
		result = daten.getStart();

		return result;
	}

	/**
	 * ermittelt, ob die Baustellendaten gültig sind.
	 *
	 * @return <code>true</code>, wenn die Daten gültig sind
	 */
	public boolean isValid() {
		return daten != null;
	}

	/**
	 * Für den Test wird davon ausgegangen, das die Baustelle simulierbar ist,
	 * d.h. es wird immer der Wert <i>true</i> geliefert.
	 */
	@Override
	public final boolean pruefeSimulierbarkeit(final boolean manuell) {
		return true;
	}

	/**
	 * setzt den Anfangszeitpunkt für die AKtivierung der Baustelle.
	 *
	 * @param zeitPunkt
	 *            der Aktivierungszeitpunkt
	 */
	public final void setAnfang(final long zeitPunkt) {
		daten.setAnfang(zeitPunkt);
	}

	/**
	 * setzt den Endzeitpunkt für die Schließung der Baustelle.
	 *
	 * @param zeitPunkt
	 *            der Schließungszeitpunkt
	 */
	public final void setEnde(final long zeitPunkt) {
		daten.setEnde(zeitPunkt);
	}

	/**
	 * Die Funktion wird bei der zyklischen oder ereignisgesteuerten Prognose
	 * einer Baustelle aufgerufen. Eine Implementierung ist für den Testfall
	 * nicht erforderlich.
	 */
	@Override
	public void simuliere() {
		// keine Implementierung
	}

	@Override
	public void simuliere(final String name, final String bemerkung, final MessQuerschnittAllgemein berechnungsMq,
			final boolean manuell) {
		simuliere();
	}
}
