/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.2 Baustellensimulation
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.simbaustelle.simbaustelle;

import com.bitctrl.Constants;

import de.bsvrz.ibv.simbaustelle.parameter.IBssParameterListener;
import de.bsvrz.ibv.simbaustelle.parameter.IBssParameterObserver;

/**
 * Definition der Parameter für den Test des Berechnungsalgorithmus für die
 * Staus einer Baustelle. Die Parameter werden nicht über den Datenverteiler
 * ermittelt, es werden lediglich die für den Test angenommenen Ausgangswerte
 * zurückgeliefert.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class TestParameter implements IBssParameterObserver {

	/**
	 * Schrittweite für die Stauprognose in Stunden.
	 */
	private static final long TEST_PROGNOSE_SCHRITTWEITE_STUNDEN = 2L;

	/**
	 * Länge ab der ein prognostizierter Stau beginnt in Metern.
	 */
	private static final int TEST_LAENGE_STAUBEGINN = 360;

	/**
	 * Länge ab der ein prognostizierter Stau aufgehoben wird in Metern.
	 */
	private static final int TEST_LAENGE_STAUAUFHEBUNG = 350;

	/**
	 * durchschnittliche Fahrzeuglänge in Metern.
	 */
	private static final int TEST_FAHRZEUG_LAENGE = 6;

	/**
	 * der Prognosehorizont in Tagen.
	 */
	private static final int TEST_PROGNOSE_HORIZONT_TAGE = 3;
	/**
	 * der vorgegebene Faktor für den Q0-Wert des Fundamentaldiagramms.
	 */
	private static final double TEST_Q0_FAKTOR = 0.5;

	/**
	 * Eie Implemantierung der Funktion ist für die Ausführung des
	 * Berechnungstests nicht notwendig.
	 */
	@Override
	public void addParameterListener(final IBssParameterListener listener) {
		// nicht notwendig
	}

	/**
	 * Das Aktualisierungsintervall speilt für den Test keine Rolle, da die
	 * Berechnung direkt ausgeführt wird. Die Funktion liefert daher immer den
	 * Wert <b><i>0</i></b> zurück.
	 */
	@Override
	public final long getAktualisierungsIntervall() {
		return 0;
	}

	/**
	 * Die Funktion liefert den für den Test vorgegebenen Q0-Faktor <i>50 %
	 * (0.5)</i> zurück.
	 */
	@Override
	public final double getFaktorQ0() {
		return TestParameter.TEST_Q0_FAKTOR;
	}

	/**
	 * Die Funktion liefert den für den Test vorgegebenen Prognosehorizont von 3
	 * Tagen.
	 */
	@Override
	public final long getHorizont() {
		return TestParameter.TEST_PROGNOSE_HORIZONT_TAGE * Constants.MILLIS_PER_DAY;
	}

	/**
	 * Die Funktion liefert die für den Test vorgegebene durchschnittliche
	 * Fahrzeuglänge von 6 Metern.
	 */
	@Override
	public final long getLaengePkw() {
		return TestParameter.TEST_FAHRZEUG_LAENGE;
	}

	/**
	 * Die Funktion liefert die für den Test vorgegebene Stauaufhebungslänge von
	 * 350 Metern.
	 */
	@Override
	public final long getLaengeStauAufhebung() {
		return TestParameter.TEST_LAENGE_STAUAUFHEBUNG;
	}

	/**
	 * Die Funktion liefert die für den Test vorgegebene Anfangsstaulänge von
	 * 360 Metern.
	 */
	@Override
	public final long getLaengeStauBeginn() {
		return TestParameter.TEST_LAENGE_STAUBEGINN;
	}

	/**
	 * Die Funktion liefert die für den Test vorgegebene Prognoseschrittweite
	 * von 2 Stunden.
	 */
	@Override
	public final long getPrognoseSchrittweite() {
		return Constants.MILLIS_PER_HOUR * TestParameter.TEST_PROGNOSE_SCHRITTWEITE_STUNDEN;
	}

	/**
	 * Die Funktion wird für den Test nicht benötigt und ist deshalb nicht
	 * implementiert.
	 */
	@Override
	public void removeParameterListener(final IBssParameterListener listener) {
		// keine Implementierung erforderlich
	}
}
